@@ -1,6 +1,8 @@
 # https://adamj.eu/tech/2021/05/13/python-type-hints-how-to-fix-circular-imports/
 from __future__ import annotations
 
+# https://gist.github.com/benkehoe/066a73903e84576a8d6d911cfedc2df6
+import importlib.metadata as importlib_metadata
 import logging
 import os
 import urllib.parse
@@ -22,7 +24,7 @@ from typing import (
 import numpy.typing as npt
 import pydantic
 import requests
-from pydantic import parse_obj_as
+from pydantic import TypeAdapter
 from segments.exceptions import (
     AlreadyExistsError,
     APILimitError,
@@ -36,7 +38,6 @@ from segments.exceptions import (
     ValidationError,
 )
 from segments.typing import (
-    AuthHeader,
     AWSFields,
     Category,
     Collaborator,
@@ -58,12 +59,18 @@ from segments.typing import (
 )
 from typing_extensions import Literal, get_args
 
+try:
+    # __package__ allows for the case where __name__ is "__main__"
+    __version__ = importlib_metadata.version(__package__ or __name__)
+except importlib_metadata.PackageNotFoundError:
+    __version__ = "0.0.0"
+
 ################################
 # Constants and type variables #
 ################################
 logger = logging.getLogger(__name__)
 T = TypeVar("T")
-VERSION = "1.0.20"
+VERSION = __version__
 
 
 ####################
@@ -77,8 +84,10 @@ def handle_exceptions(
 
     Args:
         model: The class to parse the JSON response into. Defaults to :obj:`None`.
+
     Returns:
         A wrapper function (of this exception handler decorator).
+
     Raises:
         :exc:`~segments.exceptions.ValidationError`: If validation of the response fails - catches :exc:`pydantic.ValidationError`.
         :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -102,7 +111,7 @@ def handle_exceptions(
                     if "throttled" in message:
                         raise APILimitError(message)
                 if model:
-                    m = parse_obj_as(model, r_json)
+                    m = TypeAdapter(model).validate_python(r_json)
                     return m
             return r
         except requests.exceptions.Timeout as e:
@@ -120,6 +129,8 @@ def handle_exceptions(
                 or "is already a collaborator" in text
             ):
                 raise CollaboratorError(message=text, cause=e)
+            if "authentication credentials were not provided" in text:
+                raise AuthenticationError(message=text, cause=e)
             if (
                 "cannot leave the organization" in text
                 or "need to be an administrator" in text
@@ -144,8 +155,6 @@ def handle_exceptions(
 ##########
 # Client #
 ##########
-
-
 class SegmentsClient:
     """A client with a connection to the Segments.ai platform.
 
@@ -176,6 +185,7 @@ class SegmentsClient:
     Args:
         api_key: Your Segments.ai API key. If no API key given, reads ``SEGMENTS_API_KEY`` from the environment. Defaults to :obj:`None`.
         api_url: URL of the Segments.ai API. Defaults to ``https://api.segments.ai/``.
+
     Raises:
         :exc:`~segments.exceptions.AuthenticationError`: If an invalid API key is used or (when not passing the API key directly) if ``SEGMENTS_API_KEY`` is not found in your environment.
     """
@@ -264,7 +274,12 @@ class SegmentsClient:
     ############
     # Datasets #
     ############
-    def get_datasets(self, user: Optional[str] = None) -> List[Dataset]:
+    def get_datasets(
+        self,
+        user: Optional[str] = None,
+        per_page: int = 1000,
+        page: int = 1,
+    ) -> List[Dataset]:
         """Get a list of datasets.
 
         .. code-block:: python
@@ -275,6 +290,9 @@ class SegmentsClient:
 
         Args:
             user: The user for which to get the datasets. Leave empty to get datasets of current user. Defaults to :obj:`None`.
+            per_page: Pagination parameter indicating the maximum number of datasets to return. Defaults to ``1000``.
+            page: Pagination parameter indicating the page to return. Defaults to ``1``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the datasets fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -283,10 +301,13 @@ class SegmentsClient:
             :exc:`~segments.exceptions.TimeoutError`: If the request times out.
         """
 
+        # pagination
+        query_string = f"?per_page={per_page}&page={page}"
+
         if user:
-            r = self._get(f"/users/{user}/datasets/", model=List[Dataset])
+            r = self._get(f"/users/{user}/datasets/{query_string}", model=List[Dataset])
         else:
-            r = self._get("/user/datasets/", model=List[Dataset])
+            r = self._get(f"/user/datasets/{query_string}", model=List[Dataset])
 
         return cast(List[Dataset], r)
 
@@ -301,6 +322,7 @@ class SegmentsClient:
 
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the dataset fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -317,11 +339,12 @@ class SegmentsClient:
         self,
         name: str,
         description: str = "",
-        task_type: TaskType = "segmentation-bitmap",
+        task_type: TaskType = TaskType.SEGMENTATION_BITMAP,
         task_attributes: Optional[Union[Dict[str, Any], TaskAttributes]] = None,
-        category: Category = "other",
+        category: Category = Category.OTHER,
         public: bool = False,
         readme: str = "",
+        metadata: Optional[Dict[str, Any]] = None,
         enable_skip_labeling: bool = True,
         enable_skip_reviewing: bool = False,
         enable_ratings: bool = False,
@@ -372,6 +395,7 @@ class SegmentsClient:
             category: The dataset category. Defaults to ``other``.
             public: The dataset visibility. Defaults to :obj:`False`.
             readme: The dataset readme. Defaults to ``''``.
+            metadata: Any dataset metadata. Example: ``{'day': 'sunday', 'robot_id': 3}``.
             enable_skip_labeling: Enable the skip button in the labeling workflow. Defaults to :obj:`True`.
             enable_skip_reviewing: Enable the skip button in the reviewing workflow. Defaults to :obj:`False`.
             enable_ratings: Enable star-ratings for labeled images. Defaults to :obj:`False`.
@@ -381,6 +405,7 @@ class SegmentsClient:
             enable_label_status_verified: Enable an additional label status "Verified". Defaults to :obj:`False`.
             enable_3d_cuboid_rotation: Enable 3D cuboid rotation (i.e., yaw, pitch and roll). Defaults to :obj:`False`.
             organization: The username of the organization for which this dataset should be created. None will create a dataset for the current user. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the task attributes fails.
             :exc:`~segments.exceptions.ValidationError`: If validation of the dataset fails.
@@ -398,14 +423,14 @@ class SegmentsClient:
 
         if type(task_attributes) is dict:
             try:
-                TaskAttributes.parse_obj(task_attributes)
+                TaskAttributes.model_validate(task_attributes)
             except pydantic.ValidationError as e:
                 logger.error(
                     "Did you use the right task attributes? Please refer to the online documentation: https://docs.segments.ai/reference/categories-and-task-attributes#object-attribute-format.",
                 )
                 raise ValidationError(message=str(e), cause=e)
         elif type(task_attributes) is TaskAttributes:
-            task_attributes = task_attributes.dict()
+            task_attributes = task_attributes.model_dump()
 
         payload: Dict[str, Any] = {
             "name": name,
@@ -426,6 +451,9 @@ class SegmentsClient:
             "data_type": "IMAGE",
         }
 
+        if metadata:
+            payload["metadata"] = metadata
+
         endpoint = (
             f"/organizations/{organization}/datasets/"
             if organization is not None
@@ -445,6 +473,7 @@ class SegmentsClient:
         category: Optional[Category] = None,
         public: Optional[bool] = None,
         readme: Optional[str] = None,
+        metadata: Optional[Dict[str, Any]] = None,
         enable_skip_labeling: Optional[bool] = None,
         enable_skip_reviewing: Optional[bool] = None,
         enable_ratings: Optional[bool] = None,
@@ -471,6 +500,7 @@ class SegmentsClient:
             category: The dataset category. Defaults to :obj:`None`.
             public: The dataset visibility. Defaults to :obj:`None`.
             readme: The dataset readme. Defaults to :obj:`None`.
+            metadata: Any dataset metadata. Example: ``{'day': 'sunday', 'robot_id': 3}``.
             enable_skip_labeling: Enable the skip button in the labeling workflow. Defaults to :obj:`None`.
             enable_skip_reviewing: Enable the skip button in the reviewing workflow. Defaults to :obj:`None`.
             enable_ratings: Enable star-ratings for labeled images. Defaults to :obj:`None`.
@@ -479,6 +509,7 @@ class SegmentsClient:
             enable_save_button: Enable a save button in the labeling and reviewing workflow, to save unfinished work. Defaults to :obj:`False`.
             enable_label_status_verified: Enable an additional label status "Verified". Defaults to :obj:`False`.
             enable_3d_cuboid_rotation: Enable 3D cuboid rotation (i.e., yaw, pitch and roll). Defaults to :obj:`False`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the dataset fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -497,7 +528,7 @@ class SegmentsClient:
 
         if task_attributes:
             payload["task_attributes"] = (
-                task_attributes.dict()
+                task_attributes.model_dump()
                 if type(task_attributes) is TaskAttributes
                 else task_attributes
             )
@@ -511,6 +542,9 @@ class SegmentsClient:
         if readme:
             payload["readme"] = readme
 
+        if metadata:
+            payload["metadata"] = metadata
+
         if enable_skip_labeling:
             payload["enable_skip_labeling"] = enable_skip_labeling
 
@@ -552,6 +586,7 @@ class SegmentsClient:
 
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NotFoundError`: If the dataset is not found.
@@ -567,6 +602,7 @@ class SegmentsClient:
         new_name: Optional[str] = None,
         new_task_type: Optional[TaskType] = None,
         new_public: Optional[bool] = None,
+        organization: Optional[str] = None,
     ) -> Dataset:
         """Clone a dataset.
 
@@ -588,6 +624,8 @@ class SegmentsClient:
             new_name: The dataset name for the clone. Defaults to ``f'{old_dataset_name}-clone'``.
             new_task_type: The task type for the clone. Defaults to the task type of the original dataset.
             new_public: The visibility for the clone. Defaults to the visibility of the original dataset.
+            organization: The username of the organization for which this dataset should be created. None will create a dataset for the current user. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the dataset fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -608,6 +646,9 @@ class SegmentsClient:
         if new_public:
             payload["public"] = new_public
 
+        if organization:
+            payload["owner"] = organization
+
         r = self._post(
             f"/datasets/{dataset_identifier}/clone/", data=payload, model=Dataset
         )
@@ -631,6 +672,7 @@ class SegmentsClient:
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             username: The username of the collaborator to be added.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the collaborator fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -646,7 +688,7 @@ class SegmentsClient:
         return cast(Collaborator, r)
 
     def add_dataset_collaborator(
-        self, dataset_identifier: str, username: str, role: Role = "labeler"
+        self, dataset_identifier: str, username: str, role: Role = Role.LABELER
     ) -> Collaborator:
         """Add a dataset collaborator.
 
@@ -660,7 +702,8 @@ class SegmentsClient:
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             username: The username of the collaborator to be added.
-            role: The role of the collaborator to be added. Defaults to ``labeler``.
+            role: The role of the collaborator to be added. One of ``labeler``, ``reviewer``, ``manager``, ``admin``. Defaults to ``labeler``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the collaborator fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -679,7 +722,7 @@ class SegmentsClient:
     def update_dataset_collaborator(
         self, dataset_identifier: str, username: str, role: Role
     ) -> Collaborator:
-        """ "Update a dataset collaborator.
+        """Update a dataset collaborator.
 
         .. code-block:: python
 
@@ -692,6 +735,7 @@ class SegmentsClient:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             username: The username of the collaborator to be added.
             role: The role of the collaborator to be added. Defaults to ``labeler``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the collaborator fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -722,6 +766,7 @@ class SegmentsClient:
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             username: The username of the collaborator to be deleted.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NotFoundError`: If the dataset or dataset collaborator is not found.
@@ -764,6 +809,7 @@ class SegmentsClient:
             direction: Sorting direction. One of ``asc`` (ascending) or ``desc`` (descending). Defaults to ``asc``.
             per_page: Pagination parameter indicating the maximum number of samples to return. Defaults to ``1000``.
             page: Pagination parameter indicating the page to return. Defaults to ``1``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the samples fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -810,13 +856,18 @@ class SegmentsClient:
             result.pop("label", None)
 
         try:
-            results = parse_obj_as(List[Sample], results)
+            results = TypeAdapter(List[Sample]).validate_python(results)
         except pydantic.ValidationError as e:
             raise ValidationError(message=str(e), cause=e)
 
         return cast(List[Sample], results)
 
-    def get_sample(self, uuid: str, labelset: Optional[str] = None) -> Sample:
+    def get_sample(
+        self,
+        uuid: str,
+        labelset: Optional[str] = None,
+        include_signed_url: bool = False,
+    ) -> Sample:
         """Get a sample.
 
         .. code-block:: python
@@ -828,6 +879,8 @@ class SegmentsClient:
         Args:
             uuid: The sample uuid.
             labelset: If defined, this additionally returns the label for the given labelset. Defaults to :obj:`None`.
+            include_signed_url: Whether to return the pre-signed URL in case of private S3 buckets. Defaults to :obj:`False`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the samples fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -841,6 +894,9 @@ class SegmentsClient:
         if labelset:
             query_string += f"?labelset={labelset}"
 
+        if include_signed_url:
+            query_string += "?include_signed_urls=1"
+
         r = self._get(query_string, model=Sample)
 
         return cast(Sample, r)
@@ -891,6 +947,7 @@ class SegmentsClient:
             assigned_labeler: The username of the user who should label this sample. Leave empty to not assign a specific labeler. Defaults to :obj:`None`.
             assigned_reviewer: The username of the user who should review this sample. Leave empty to not assign a specific reviewer. Defaults to :obj:`None`.
             embedding: Embedding of this sample represented by an array of floats.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the sample attributes fails.
             :exc:`~segments.exceptions.ValidationError`: If validation of the sample fails.
@@ -903,14 +960,14 @@ class SegmentsClient:
 
         if type(attributes) is dict:
             try:
-                parse_obj_as(SampleAttributes, attributes)
+                TypeAdapter(SampleAttributes).validate_python(attributes)
             except pydantic.ValidationError as e:
                 logger.error(
                     "Did you use the right sample attributes? Please refer to the online documentation: https://docs.segments.ai/reference/sample-and-label-types/sample-types.",
                 )
                 raise ValidationError(message=str(e), cause=e)
         elif type(attributes) in get_args(SampleAttributes):
-            attributes = attributes.dict()
+            attributes = attributes.model_dump()
 
         payload: Dict[str, Any] = {
             "name": name,
@@ -949,6 +1006,7 @@ class SegmentsClient:
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: jane/flowers.
             samples: A list of dicts with required ``name``, ``attributes`` fields and optional ``metadata``, ``priority``, ``embedding`` fields. See :meth:`.add_sample` for details.
+
         Raises:
             :exc:`KeyError`: If 'name' or 'attributes' is not in a sample dict.
             :exc:`~segments.exceptions.ValidationError`: If validation of the attributes of a sample fails.
@@ -969,14 +1027,14 @@ class SegmentsClient:
                     )
 
                 try:
-                    parse_obj_as(SampleAttributes, sample["attributes"])
+                    TypeAdapter(SampleAttributes).validate_python(sample["attributes"])
                 except pydantic.ValidationError as e:
                     logger.error(
                         "Did you use the right sample attributes? Please refer to the online documentation: https://docs.segments.ai/reference/sample-and-label-types/sample-types.",
                     )
                     raise ValidationError(message=str(e), cause=e)
             elif type(sample) is Sample:
-                sample = sample.dict()
+                sample = sample.model_dump()
 
         payload = samples
 
@@ -1022,6 +1080,7 @@ class SegmentsClient:
             assigned_labeler: The username of the user who should label this sample. Leave empty to not assign a specific labeler.
             assigned_reviewer: The username of the user who should review this sample. Leave empty to not assign a specific reviewer.
             embedding: Embedding of this sample represented by list of floats.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.ValidationError`: If validation of the samples fails.
@@ -1037,7 +1096,7 @@ class SegmentsClient:
 
         if attributes:
             payload["attributes"] = (
-                attributes.dict()
+                attributes.model_dump()
                 if type(attributes) in get_args(SampleAttributes)
                 else attributes
             )
@@ -1072,6 +1131,7 @@ class SegmentsClient:
 
         Args:
             uuid: The sample uuid.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NotFoundError`: If the sample is not found.
@@ -1087,6 +1147,9 @@ class SegmentsClient:
     def get_label(self, sample_uuid: str, labelset: str = "ground-truth") -> Label:
         """Get a label.
 
+        Note:
+            If the sample is unlabeled, a :exc:`~segments.exceptions.NotFoundError` will be raised.
+
         .. code-block:: python
 
             sample_uuid = '602a3eec-a61c-4a77-9fcc-3037ce5e9606'
@@ -1096,10 +1159,11 @@ class SegmentsClient:
         Args:
             sample_uuid: The sample uuid.
             labelset: The labelset this label belongs to. Defaults to ``ground-truth``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the label fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
-            :exc:`~segments.exceptions.NotFoundError`: If the sample or labelset is not found.
+            :exc:`~segments.exceptions.NotFoundError`: If the sample or labelset is not found or if the sample is unlabeled.
             :exc:`~segments.exceptions.NetworkError`: If the request is not valid or if the server experienced an error.
             :exc:`~segments.exceptions.TimeoutError`: If the request times out.
         """
@@ -1113,7 +1177,7 @@ class SegmentsClient:
         sample_uuid: str,
         labelset: str,
         attributes: Union[Dict[str, Any], LabelAttributes],
-        label_status: LabelStatus = "PRELABELED",
+        label_status: LabelStatus = LabelStatus.PRELABELED,
         score: Optional[float] = None,
     ) -> Label:
         """Add a label to a sample.
@@ -1148,6 +1212,7 @@ class SegmentsClient:
             attributes: The label attributes. Please refer to the `online documentation <https://docs.segments.ai/reference/sample-and-label-types/label-types>`__.
             label_status: The label status. Defaults to ``PRELABELED``.
             score: The label score. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the attributes fails.
             :exc:`~segments.exceptions.ValidationError`: If validation of the label fails.
@@ -1159,14 +1224,14 @@ class SegmentsClient:
 
         if type(attributes) is dict:
             try:
-                parse_obj_as(LabelAttributes, attributes)
+                TypeAdapter(LabelAttributes).validate_python(attributes)
             except pydantic.ValidationError as e:
                 logger.error(
                     "Did you use the right label attributes? Please refer to the online documentation: https://docs.segments.ai/reference/sample-and-label-types/label-types.",
                 )
                 raise ValidationError(message=str(e), cause=e)
         elif type(attributes) in get_args(LabelAttributes):
-            attributes = attributes.dict()
+            attributes = attributes.model_dump()
 
         payload: Dict[str, Any] = {
             "label_status": label_status,
@@ -1184,8 +1249,8 @@ class SegmentsClient:
         self,
         sample_uuid: str,
         labelset: str,
-        attributes: Union[Dict[str, Any], LabelAttributes],
-        label_status: LabelStatus = "PRELABELED",
+        attributes: Optional[Union[Dict[str, Any], LabelAttributes]] = None,
+        label_status: LabelStatus = LabelStatus.PRELABELED,
         score: Optional[float] = None,
     ) -> Label:
         """Update a label.
@@ -1212,9 +1277,10 @@ class SegmentsClient:
         Args:
             sample_uuid: The sample uuid.
             labelset: The labelset this label belongs to.
-            attributes: The label attributes. Please refer to the `online documentation <https://docs.segments.ai/reference/sample-and-label-types/label-types>`__.
+            attributes: The label attributes. Please refer to the `online documentation <https://docs.segments.ai/reference/sample-and-label-types/label-types>`__. Defaults to :obj:`None`.
             label_status: The label status. Defaults to ``PRELABELED``.
             score: The label score. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the label fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1227,7 +1293,7 @@ class SegmentsClient:
 
         if attributes:
             payload["attributes"] = (
-                attributes.dict()
+                attributes.model_dump()
                 if type(attributes) in get_args(LabelAttributes)
                 else attributes
             )
@@ -1254,6 +1320,7 @@ class SegmentsClient:
         Args:
             sample_uuid: The sample uuid.
             labelset: The labelset this label belongs to.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NotFoundError`: If the sample or labelset is not found.
@@ -1278,6 +1345,7 @@ class SegmentsClient:
 
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the labelsets fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1305,6 +1373,7 @@ class SegmentsClient:
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             name: The name of the labelset.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the labelset fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1334,6 +1403,7 @@ class SegmentsClient:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             name: The name of the labelset.
             description: The labelset description. Defaults to ``''``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the labelset fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1367,6 +1437,7 @@ class SegmentsClient:
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             name: The name of the labelset.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NotFoundError`: If the dataset or labelset is not found.
@@ -1391,6 +1462,7 @@ class SegmentsClient:
 
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NotFoundError`: If the dataset is not found.
@@ -1406,7 +1478,7 @@ class SegmentsClient:
         self,
         sample_uuid: str,
         description: str,
-        status: IssueStatus = "OPEN",
+        status: IssueStatus = IssueStatus.OPEN,
     ) -> Issue:
         """Add an issue to a sample.
 
@@ -1421,6 +1493,7 @@ class SegmentsClient:
             sample_uuid: The sample uuid.
             description: The issue description.
             status: The issue status. One of ``OPEN`` or ``CLOSED``. Defaults to ``OPEN``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the issue fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1457,6 +1530,7 @@ class SegmentsClient:
             uuid: The issue uuid.
             description: The issue description. Defaults to :obj:`None`.
             status: The issue status. One of ``OPEN`` or ``CLOSED``. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the issue fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1487,6 +1561,7 @@ class SegmentsClient:
 
         Args:
             uuid: The issue uuid.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NotFoundError`: If the issue is not found.
@@ -1511,6 +1586,7 @@ class SegmentsClient:
 
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the releases fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1536,6 +1612,7 @@ class SegmentsClient:
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             name: The name of the release.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the release fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1565,6 +1642,7 @@ class SegmentsClient:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             name: The name of the release.
             description: The release description. Defaults to ``''``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the release fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1595,6 +1673,7 @@ class SegmentsClient:
         Args:
             dataset_identifier: The dataset identifier, consisting of the name of the dataset owner followed by the name of the dataset itself. Example: ``jane/flowers``.
             name: The name of the release.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NotFoundError`: If the dataset or release is not found.
@@ -1624,6 +1703,7 @@ class SegmentsClient:
         Args:
             file: A file object.
             filename: The file name. Defaults to ``label.png``.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the file fails.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1632,7 +1712,7 @@ class SegmentsClient:
         """
 
         r = self._post("/assets/", data={"filename": filename})
-        presigned_post_fields = PresignedPostFields.parse_obj(
+        presigned_post_fields = PresignedPostFields.model_validate(
             r.json()["presignedPostFields"]
         )
         self._upload_to_aws(
@@ -1640,7 +1720,7 @@ class SegmentsClient:
         )
 
         try:
-            f = File.parse_obj(r.json())
+            f = File.model_validate(r.json())
         except pydantic.ValidationError as e:
             raise ValidationError(message=str(e), cause=e)
 
@@ -1662,6 +1742,7 @@ class SegmentsClient:
             endpoint: The API endpoint.
             auth: If we want to authorize the request. Defaults to :obj:`True`.
             model: The class to parse the JSON response into. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the response fails - catches :exc:`pydantic.ValidationError`.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
@@ -1669,7 +1750,7 @@ class SegmentsClient:
             :exc:`~segments.exceptions.TimeoutError`: If the request times out - catches :exc:`requests.exceptions.TimeoutError`.
         """
 
-        headers = self._get_auth_header() if auth else None
+        headers = self._get_headers(auth)
 
         r = self.api_session.get(
             urllib.parse.urljoin(self.api_url, endpoint), headers=headers
@@ -1692,13 +1773,14 @@ class SegmentsClient:
             data: The JSON data. Defaults to :obj:`None`.
             auth: If we want to authorize the request with the API key. Defaults to :obj:`True`.
             model: The class to parse the JSON response into. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the response fails - catches :exc:`pydantic.ValidationError`.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NetworkError`: If the request is not valid or if the server experienced an error - catches :exc:`requests.HTTPError` and catches :exc:`requests.RequestException`.
             :exc:`~segments.exceptions.TimeoutError`: If the request times out - catches :exc:`requests.exceptions.TimeoutError`.
         """
-        headers = self._get_auth_header() if auth else None
+        headers = self._get_headers(auth)
 
         r = self.api_session.post(
             urllib.parse.urljoin(self.api_url, endpoint),
@@ -1723,13 +1805,14 @@ class SegmentsClient:
             data: The JSON data. Defaults to :obj:`None`.
             auth: If we want to authorize the request with the API key. Defaults to :obj:`True`.
             model: The class to parse the JSON response into. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the response fails - catches :exc:`pydantic.ValidationError`.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NetworkError`: If the request is not valid or if the server experienced an error - catches :exc:`requests.HTTPError` and catches :exc:`requests.RequestException`.
             :exc:`~segments.exceptions.TimeoutError`: If the request times out - catches :exc:`requests.exceptions.TimeoutError`.
         """
-        headers = self._get_auth_header() if auth else None
+        headers = self._get_headers(auth)
 
         r = self.api_session.put(
             urllib.parse.urljoin(self.api_url, endpoint),
@@ -1754,13 +1837,14 @@ class SegmentsClient:
             data: The JSON data. Defaults to :obj:`None`.
             auth: If we want to authorize the request with the API key. Defaults to :obj:`True`.
             model: The class to parse the JSON response into. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the response fails - catches :exc:`pydantic.ValidationError`.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NetworkError`: If the request is not valid or if the server experienced an error - catches :exc:`requests.HTTPError` and catches :exc:`requests.RequestException`.
             :exc:`~segments.exceptions.TimeoutError`: If the request times out - catches :exc:`requests.exceptions.TimeoutError`.
         """
-        headers = self._get_auth_header() if auth else None
+        headers = self._get_headers(auth)
 
         r = self.api_session.patch(
             urllib.parse.urljoin(self.api_url, endpoint),
@@ -1785,13 +1869,14 @@ class SegmentsClient:
             data: The JSON data. Defaults to :obj:`None`.
             auth: If we want to authorize the request with the API key. Defaults to :obj:`True`.
             model: The class to parse the JSON response into. Defaults to :obj:`None`.
+
         Raises:
             :exc:`~segments.exceptions.ValidationError`: If validation of the response fails - catches :exc:`pydantic.ValidationError`.
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NetworkError`: If the request is not valid or if the server experienced an error - catches :exc:`requests.HTTPError` and catches :exc:`requests.RequestException`.
             :exc:`~segments.exceptions.TimeoutError`: If the request times out - catches :exc:`requests.exceptions.TimeoutError`.
         """
-        headers = self._get_auth_header() if auth else None
+        headers = self._get_headers(auth)
 
         r = self.api_session.delete(
             urllib.parse.urljoin(self.api_url, endpoint),
@@ -1801,9 +1886,12 @@ class SegmentsClient:
 
         return r
 
-    def _get_auth_header(self) -> Optional[AuthHeader]:
+    def _get_headers(self, auth: bool = True) -> Dict[str, str]:
         """Get the authorization header with the API key."""
-        return {"Authorization": f"APIKey {self.api_key}"} if self.api_key else None
+        headers = {"X-source": "python-sdk"}
+        if auth and self.api_key:
+            headers["Authorization"] = f"APIKey {self.api_key}"
+        return headers
 
     @handle_exceptions
     def _upload_to_aws(
@@ -1815,6 +1903,7 @@ class SegmentsClient:
             file: The file we want to upload.
             url: The request's url.
             aws_fields: The AWS fields.
+
         Raises:
             :exc:`~segments.exceptions.APILimitError`: If the API limit is exceeded.
             :exc:`~segments.exceptions.NetworkError`: If the request is not valid or if the server experienced an error.
