@@ -10,8 +10,8 @@ from urllib.parse import urlparse
 import numpy as np
 import numpy.typing as npt
 import requests
-from PIL import Image
-from pydantic import parse_obj_as
+from PIL import Image, UnidentifiedImageError
+from pydantic import TypeAdapter
 from segments.typing import LabelStatus, Release, SegmentsDatasetCategory
 from segments.utils import (
     handle_exif_rotation,
@@ -78,6 +78,7 @@ class SegmentsDataset:
         segments_dir: The directory where the data will be downloaded to for caching. Set to :obj:`None` to disable caching. Defaults to ``segments``.
         preload: Whether the data should be pre-downloaded when the dataset is initialized. Ignored if ``segments_dir`` is :obj:`None`. Defaults to :obj:`True`.
         s3_client: A boto3 S3 client, e.g. ``s3_client = boto3.client("s3")``. Needs to be provided if your images are in a private S3 bucket. Defaults to :obj:`None`.
+
     Raises:
         :exc:`ValueError`: If the release task type is not one of: ``segmentation-bitmap``, ``segmentation-bitmap-highres``, ``image-vector-sequence``, ``bboxes``, ``vector``, ``pointcloud-cuboid``, ``pointcloud-cuboid-sequence``, ``pointcloud-segmentation``, ``pointcloud-segmentation-sequence``, ``text-named-entities``, or ``text-span-categorization``.
         :exc:`ValueError`: If there is no labelset with this name.
@@ -94,7 +95,6 @@ class SegmentsDataset:
         preload: bool = True,
         s3_client: Optional[Any] = None,
     ):
-
         self.labelset = labelset
         if isinstance(filter_by, list):
             filter_by = [f.upper() for f in filter_by]
@@ -149,7 +149,7 @@ class SegmentsDataset:
             "pointcloud-segmentation",
         ]:
             raise ValueError(
-                "You can only create a dataset for tasks of type 'segmentation-bitmap', 'segmentation-bitmap-highres', 'vector', 'bboxes', 'keypoints', 'image-vector-sequence', 'pointcloud-cuboid', 'pointcloud-segmentation' for now."
+                f"You can only create a dataset for tasks of type 'segmentation-bitmap', 'segmentation-bitmap-highres', 'vector', 'bboxes', 'keypoints', 'image-vector-sequence', 'pointcloud-cuboid', 'pointcloud-segmentation' for now. Got {self.task_type}."
             )
 
         self.load_dataset()
@@ -224,7 +224,6 @@ class SegmentsDataset:
     def _load_image_from_cache(
         self, sample: Dict[str, Any]
     ) -> Tuple[Optional[Image.Image], str]:
-
         sample_name = os.path.splitext(sample["name"])[0]
         image_url = sample["attributes"]["image"]["url"]
         image_url_parsed = urlparse(image_url)
@@ -242,7 +241,13 @@ class SegmentsDataset:
                         image_url, image_filename, self.s3_client
                     )
                 else:
-                    image = Image.open(image_filename)
+                    try:
+                        image = Image.open(image_filename)
+                    except UnidentifiedImageError:
+                        image = None
+                        logger.error(
+                            f"Something went wrong loading image: {image_filename}"
+                        )
             else:
                 image = load_image_from_url(image_url, self.s3_client)
 
@@ -253,12 +258,14 @@ class SegmentsDataset:
     def _load_segmentation_bitmap_from_cache(
         self, sample: Dict[str, Any], labelset: str
     ) -> Union[npt.NDArray[np.uint32], Image.Image]:
-
         sample_name = os.path.splitext(sample["name"])[0]
         label = sample["labels"][labelset]
         segmentation_bitmap_url = label["attributes"]["segmentation_bitmap"]["url"]
         url_extension = os.path.splitext(urlparse(segmentation_bitmap_url).path)[1]
 
+        if not segmentation_bitmap_url:
+            return None
+
         if self.caching_enabled:
             # segmentation_bitmap_filename = os.path.join(self.image_dir, '{}{}'.format(label['uuid'], url_extension))
             segmentation_bitmap_filename = os.path.join(
@@ -276,8 +283,7 @@ class SegmentsDataset:
 
     @property
     def categories(self) -> List[SegmentsDatasetCategory]:
-        return parse_obj_as(
-            List[SegmentsDatasetCategory],
+        return TypeAdapter(List[SegmentsDatasetCategory]).validate_python(
             self.release["dataset"]["task_attributes"]["categories"],
         )
         # categories = {}
@@ -299,9 +305,10 @@ class SegmentsDataset:
             return sample
 
         # Load the image
+        image, image_filename = None, None
         try:
             image, image_filename = self._load_image_from_cache(sample)
-        except TypeError:
+        except (KeyError, TypeError):
             logger.error(
                 f"Something went wrong loading sample {sample['name']}: {sample}"
             )
@@ -334,7 +341,7 @@ class SegmentsDataset:
                         "attributes": attributes,
                     }
                 )
-            except TypeError:
+            except (KeyError, TypeError):
                 item.update(
                     {
                         "segmentation_bitmap": None,
