@@ -19,21 +19,32 @@ class SegmentsError(Exception):
         return self.message
 
 
+##################
+# Authentication #
+##################
 class AuthenticationError(SegmentsError):
     """Raised when an API key fails authentication."""
 
     pass
 
 
+#################
+# Authorization #
+#################
 class AuthorizationError(SegmentsError):
     """Raised when a user is unauthorized to perform the given request."""
 
     pass
 
 
+########################
+# Network (HTTP error) #
+########################
 class NetworkError(SegmentsError):
     """Raised when an HTTP error occurs."""
 
+    pass
+
 
 class NotFoundError(NetworkError):
     """Raised when the requested object is not found (e.g., because the name is misspelled)."""
@@ -47,18 +58,39 @@ class AlreadyExistsError(NetworkError):
     pass
 
 
+class CollaboratorError(NetworkError):
+    """Raised when a collaborator error occurs (e.g., a collaborator already has a role / cannot be added to an organization / ...)."""
+
+    pass
+
+
+class SubscriptionError(NetworkError):
+    """Raised when a subscription error occurs (e.g., your trial ends, you exceed your user limit for private datasets, ...)."""
+
+    pass
+
+
+############
+# Time out #
+############
 class TimeoutError(SegmentsError):
     """Raised when a request times out."""
 
     pass
 
 
+#############
+# API limit #
+#############
 class APILimitError(SegmentsError):
     """Raised when the user performs too many requests in a period of time."""
 
     pass
 
 
+##############
+# Validation #
+##############
 class ValidationError(SegmentsError):
     """Raised when validation of the response fails."""
 
