@@ -126,6 +126,7 @@ def rgb2id(color: Union[npt.NDArray[Any], RGB]) -> Union[npt.NDArray[Any], int]:
 
     Args:
         color: An RGB value.
+
     Returns:
         The id.
     """
@@ -142,6 +143,7 @@ def id2rgb(id_map: npt.NDArray[Any]) -> Union[npt.NDArray[Any], RGB]:
 
     Args:
         id_map: An id map.
+
     Returns:
         An rgb.
     """
@@ -168,7 +170,6 @@ def get_color(id: int) -> RGB:
 def colorize(
     img: npt.NDArray[Any], colormap: Optional[ColorMap] = None
 ) -> npt.NDArray[Any]:
-
     indices = np.unique(img)
     indices = indices[indices != 0]
 
@@ -190,6 +191,7 @@ def get_bbox(binary_mask: npt.NDArray[Any]) -> Union[Tuple[int, int, int, int],
 
     Args:
         binary_mask: The binary mask as a numpy array.
+
     Returns:
         The bounding box or False if no bounding box is found.
     """
@@ -213,8 +215,10 @@ def export_coco_instance(
     Args:
         dataset: A :class:`.SegmentsDataset`.
         export_folder: The output directory.
+
     Returns:
         Returns the file name and the image directory name.
+
     Raises:
         :exc:`ImportError`: If pycocotools is not installed.
     """
@@ -371,7 +375,7 @@ def export_coco_instance(
 
     json_data = {
         "info": info,
-        "categories": [category.dict() for category in categories],
+        "categories": [category.model_dump() for category in categories],
         "images": images,
         "annotations": annotations
         # "segment_info": [] # Only in Panoptic annotations
@@ -398,6 +402,7 @@ def export_coco_panoptic(
     Args:
         dataset: A :class:`.SegmentsDataset`.
         export_folder: The output directory.
+
     Returns:
         Returns the file name and the image directory name.
     """
@@ -549,7 +554,7 @@ def export_coco_panoptic(
     # PUT EVERYTHING TOGETHER
     json_data = {
         "info": info,
-        "categories": [category.dict() for category in categories],
+        "categories": [category.model_dump() for category in categories],
         "images": images,
         "annotations": annotations,
     }
@@ -581,6 +586,7 @@ def export_image(
         export_folder: The output directory.
         export_format: The export format.
         id_increment: Increment the category ids with this number.
+
     Returns:
         Returns the export folder.
     """
@@ -600,7 +606,7 @@ def export_image(
         )
 
         categories.append(
-            SegmentsDatasetCategory.parse_obj(
+            SegmentsDatasetCategory.model_validate(
                 {
                     "id": category.id,
                     "name": category.name,
@@ -686,8 +692,10 @@ def export_yolo(
         export_folder: The output directory.
         image_width: The width of the image (needed for ``image-vector-sequence``).
         image_height: The height of the image (needed for ``image-vector-sequence``).
+
     Returns:
         The image directory of the dataset.
+
     Raises:
         :exc:`ValueError`: If the dataset is not a bounding box dataset.
         :exc:`ValueError`: If the dataset is an ``image-vector-sequence``and the image width or image height is :obj:`None`.
@@ -715,9 +723,7 @@ def export_yolo(
                     # Save it to the file
                     # print(category_id, x_center, y_center, width, height)
                     f.write(
-                        "{} {:.6f} {:.6f} {:.6f} {:.6f}\n".format(
-                            category_id, x_center, y_center, width, height
-                        )
+                        f"{category_id} {x_center:.6f} {y_center:.6f} {width:.6f} {height:.6f}\n"
                     )
 
     # Create export folder
@@ -769,17 +775,23 @@ def export_yolo(
                 export_folder, dataset.image_dir, f"{image_name}.txt"
             )
 
-            if image_width is None or image_height is None:
-                image_width = sample["image"].width
-                image_height = sample["image"].height
-
             if "annotations" in sample and sample["annotations"]:
                 annotations = sample["annotations"]
                 write_yolo_file(
                     file_name,
                     annotations,
-                    cast(float, image_width),
-                    cast(float, image_height),
+                    cast(
+                        float,
+                        image_width
+                        if image_width is not None
+                        else sample["image"].width,
+                    ),
+                    cast(
+                        float,
+                        image_height
+                        if image_height is not None
+                        else sample["image"].height,
+                    ),
                 )
 
     print(f"Exported. Images and labels in {dataset.image_dir}")
@@ -794,8 +806,10 @@ def export_polygon(
     Args:
         dataset: A :class:`.SegmentsDataset`.
         export_folder: The output directory.
+
     Raises:
         :exc:`ImportError`: If OpenCV is not installed.
+
     Returns:
         Returns the file name and the image directory name.
     """
@@ -831,10 +845,14 @@ def export_polygon(
             }
         )
 
+        segmentation_bitmap = np.array(sample["segmentation_bitmap"], np.uint32)        
         # https://scikit-image.org/docs/dev/api/skimage.measure.html#skimage.measure.regionprops
-        regions = regionprops(np.array(sample["segmentation_bitmap"], np.uint32))
+        regions = regionprops(segmentation_bitmap)
+
         regions = {region.label: region for region in regions}
 
+        segmentation_bitmap_uint8 = segmentation_bitmap.astype(np.uint8)
+
         for instance in sample["annotations"]:
             category_id = instance["category_id"]
 
@@ -853,7 +871,7 @@ def export_polygon(
                 continue
 
             instance_mask = (
-                np.asarray(sample["segmentation_bitmap"], np.uint8) == instance["id"]
+                segmentation_bitmap_uint8 == instance["id"]
             )
 
             # Allowed OpenCV data types: https://stackoverflow.com/questions/12785121/access-opencv-matrix-cv-32s-element
@@ -875,7 +893,7 @@ def export_polygon(
             annotation_id += 1
 
     json_data = {
-        "categories": [category.dict() for category in categories],
+        "categories": [category.model_dump() for category in categories],
         "images": images,
         "annotations": annotations,
     }
